; superdither ecm editor
; ---
; screen editor in ecm mode with the super dither charset.
;
; active keys:
; cursor keys		: move cursor
; spacebar		: draw char
; f1/f2/f3/help		: change color $ff15/16/17/18
; f4/f5/f6/f7		: change luminance $ff15/16/17/18
; 9			: change border color
; shift+9		: change border luminance 
; 0			: change screen color
; shift+0		: change screen luminance
; 1-8			: choose fixed chars $00/08/10/18/20/28/30/38
; z/x/c/v		: switch to $ff15/16/17/18
; shift+up/down		: change from $ff15 to $ff18
; shift+left/right	: decrease/increase char value
; commodore+clr/home	: clear screen
; return		: copy both char and color
; esc			: copy screen+color memory into $2800
; h			: switch help screen on/off
; --- 
curslo	= $c8
curshi	= $c9
cursli	= $ca
colorlo	= $ea
colorhi	= $eb
backchar= $d8
backcol	= $d9
pickchar= $da
pickcol	= $db
; ---

	org $1001-2
	dw  $1001
	
	dw nextln,0	; second word is line number
	db $9e
	if start > 9999
	db "0"+start/10000
	endif
	db "0"+(start/1000)%10,"0"+(start/100)%10,"0"+(start/10)%10,"0"+start%10,0
nextln	db 0,0
	
start
	sei
	sta $ff3e	; rom
	;
	lda #$5b	; ecm mode
	sta $ff06
	lda $ff07	; pal/ntsc friendly
	and #$40
	ora #$08	; use $18 for multicolor
	sta $ff07
	lda $ff12
	and #$03	; keep lowest 2 bits (sound)	
	ora #$c0	; $08 = $2000, $10 = $4000, etc.
	sta $ff12
	lda #chars >> 8
	sta $ff13
	;
; initial colors
	lda #$10
	sta pickchar	
	lda #$51
	sta pickcol
	lda #$42
	sta $ff15
	lda #$46
	sta $ff16
	lda #$5f
	sta $ff17
	lda #$77
	sta $ff18
	lda #$00
	sta $ff19
	;
	sta $055f
	sta $0560
	sta $0561
	sta $0562
	sta $0563
	sta $0564
	sta $0565
	sta $0566
	lda #$71
	sta $053b
;	LDA #$40	; keys repeat off
;	STA $0540
	lda #$ff	; neither screen scroll nor row down
	sta $07e9
	;
	jsr key_h	; start with help screen
	jsr fill_screen
	;
	ldx #irq & 255	; irq
	ldy #irq >> 8
	stx $0312
	sty $0313
	cli
	; preventing READY. prompt	
	jsr $d89a	; Home
	jsr $de5e	; ESC-T
	jsr $de60	; ESC-B 
	rts	; back to basic
; ---------------------

irq
	pha
	txa
	pha
	tya
	pha
	inc $ff09

; initial check for clr/home
	lda flag
	bpl loop_k-2
	lda #$7f
	sta $fd30
	sta $ff08
	lda $ff08
	cmp #$dd
	beq cbmhome
	lda #$00
	sta cntr
	dec flag
	lda tmp
	sta $ff19
	jmp blink
	;
	
cbmhome
	lda $ff19	
        clc
        adc #$10
        sta $ff19
	inc cntr
	lda cntr
	cmp #$07
	bmi keyjmp+3
	lda #$00
	sta cntr
	dec flag
	lda tmp
	sta $ff19
	;
	jsr fill_screen
	jmp irqout
	
fill_screen	
	ldx #$04	; Number of pages
	ldy #$08	; Hi byte of address ($0800)
	lda pickcol	; Byte to fill
	sta backcol
	jsr $C5A7
	ldx #$04	; This clears the screen 
	ldy #$0C	; $0C00
	lda #$20	; Space character 
	sta backchar
	jsr $C5A7
	rts

; keyboard check
	ldx #(keyb2-keyb1-1)
loop_k		
	lda keyb1,x
	sta $fd30
	sta $ff08
	lda $ff08
	cmp keyb2,x
	beq keypush
	dex
	bpl loop_k
	bmi blink	; jmp blink

; key pressed
keypush		
	txa
	asl a
	tax
	lda keyb2x,x
	sta keyjmp+1
	lda keyb2x+1,x
	sta keyjmp+2
keyjmp
	jsr $cafe
	jsr loop_kdelay
; exit irq with no kernel
blink
	inc cntr+1
	lda cntr+1
	and #$0f
	beq go_on
	cmp #$08
	beq go_off
	bne irqout
go_on
	jsr flashon
	jmp irqout
go_off
	jsr flashoff
	;	
irqout	pla
	tay
	pla
	tax
	pla
	jmp $fcbe
	;

loop_kdelay
	ldx #$07
	lda #$fc
	cmp $ff1d
	bne *-3
	cmp $ff1d
	beq *-3
	dex
	bpl loop_kdelay+2
	rts
		
flashon
	ldy cursli
	lda pickchar
	sta (curslo),y
	lda pickcol
	sta (colorlo),y
	rts
flashoff
	ldy cursli
	lda backchar
	sta (curslo),y
	lda backcol
	sta (colorlo),y
	rts
store
	ldy cursli
	lda (curslo),y
	sta backchar
	lda (colorlo),y
	sta backcol
	rts
	;
		
key_instdel
key_shinstdel
key_shhome
	rts	; no return!
key_f1
	db $af,$15,$ff		; lax $ff15
	and #$f0
	sta tmp
	inx
	txa
	and #$0f
	ora tmp
	sta $ff15
	jsr flashon
	rts
key_f2
	db $af,$16,$ff		; lax $ff16
	and #$f0
	sta tmp
	inx
	txa
	and #$0f
	ora tmp
	sta $ff16
	jsr flashon
	rts	
key_f3
	db $af,$17,$ff		; lax $ff17
	and #$f0
	sta tmp
	inx
	txa
	and #$0f
	ora tmp
	sta $ff17
	jsr flashon
	rts
key_help
	db $af,$18,$ff		; lax $ff18
	and #$f0
	sta tmp
	inx
	txa
	and #$0f
	ora tmp
	sta $ff18
	jsr flashon
	rts
key_f4
	lda $ff15
	clc
	adc #$10
	sta $ff15
	jsr flashon
	rts
key_f5
	lda $ff16
	clc
	adc #$10
	sta $ff16
	jsr flashon
	rts	
key_f6
	lda $ff17
	clc
	adc #$10
	sta $ff17
	jsr flashon
	rts
key_f7
	lda $ff18
	clc
	adc #$10
	sta $ff18
	jsr flashon
	rts
key_9
	db $af,$19,$ff		; lax $ff19
	and #$f0
	sta tmp
	inx
	txa
	and #$0f
	ora tmp
	sta $ff19
	rts
key_0
	db $a7,pickcol		; LAX $XX
	and #$f0
	sta tmp
	inx
	txa
	and #$0f
	ora tmp
	sta pickcol
	jsr flashon
	rts
key_sh9
	lda $ff19
	clc
	adc #$10
	sta $ff19
	rts
key_sh0
	lda pickcol
	clc
	adc #$10
	sta pickcol
	jsr flashon
	rts
key_space
	lda pickchar
;	ora cset
;	sta pickchar
	sta backchar
	lda pickcol
	sta backcol
	rts
key_z
	lda pickchar
	and #$3f
	sta pickchar
	rts
key_x
	lda pickchar
	and #$3f
	ora #$40
	sta pickchar
	rts
key_c
	lda pickchar
	and #$3f
	ora #$80
	sta pickchar
	rts
key_v
	lda pickchar
	and #$3f
	ora #$c0
	sta pickchar
	rts
key_left
	jsr flashoff
	ldy cursli
	beq *+4
	dey
	db $2c
	ldy #$27
	sty cursli
	jsr store
	jsr flashon
	rts
key_right
	jsr flashoff
	ldy cursli
	cpy #$27
	beq *+4
	iny
	db $2c
	ldy #$00
	sty cursli
	jsr store
	jsr flashon
	rts	
key_up
	jsr flashoff
	lda curshi
	cmp #$0c
	bne *+6
	lda curslo
	beq wrap_up
	lda curslo
	sec
	sbc #$28
	sta curslo
	sta colorlo
	lda curshi
	sbc #$00
	sta curshi
	and #$0b
	sta colorhi
	jsr store
	jsr flashon
	rts
wrap_up	
	lda #$c0
	sta curslo
	sta colorlo
	lda #$0f
	sta curshi
	and #$0b
	sta colorhi
	jsr store
	jsr flashon
	rts	
key_down
	jsr flashoff
	lda curshi
	cmp #$0f
	bne *+8
	lda curslo
	cmp #$c0
	beq wrap_down
	lda curslo
	clc
	adc #$28
	sta curslo
	sta colorlo
	lda curshi
	adc #$00
	sta curshi
	and #$0b
	sta colorhi
	jsr store
	jsr flashon
	rts
wrap_down
	lda #$00
	sta curslo
	sta colorlo
	lda #$0c
	sta curshi
	and #$0b
	sta colorhi
	jsr store
	jsr flashon
	rts
key_cbmhome
	lda $ff19
	sta tmp
	lda #$01
	sta $ff19
	inc flag
	rts
key_shright
	db $a7,pickchar		; LAX pickchar
	and #$3f
	cmp #$3f
	beq *+5
	inx
	stx pickchar
	jsr flashon
	rts
key_shleft
	db $a7,pickchar		; LAX pickchar
	and #$3f
	beq *+5
	dex
	stx pickchar
	jsr flashon
	rts
key_shdown
	db $a7,pickchar		; LAX pickchar
	and #$c0
	beq *+8
	txa
	sec
	sbc #$40
	sta pickchar
	rts
key_shup
	db $a7,pickchar		; LAX pickchar
	cmp #$c0
	bcs *+7
	clc
	adc #$40
	sta pickchar
	rts
key_home
	jsr flashoff
	lda #$00
	sta curslo
	sta cursli
	sta colorlo
	sta $ff0d
	lda #$0c
	sta curshi
	and #$08
	sta colorhi
	lda #$fc
	sta $ff0c
	jsr store
	rts
key_1
	lda pickchar
	and #$c0
	sta pickchar
	rts
key_2
	lda pickchar
	and #$c0
	ora #$08
	sta pickchar
	rts
key_3
	lda pickchar
	and #$c0
	ora #$10
	sta pickchar
	rts
key_4
	lda pickchar
	and #$c0
	ora #$18
	sta pickchar
	rts
key_5
	lda pickchar
	and #$c0
	ora #$20
	sta pickchar
	rts				
key_6
	lda pickchar
	and #$c0
	ora #$28
	sta pickchar
	rts
key_7
	lda pickchar
	and #$c0
	ora #$30
	sta pickchar
	rts
key_8
	lda pickchar
	and #$c0
	ora #$38
	sta pickchar
	rts
	
key_return
	lda backcol
	sta pickcol
	lda backchar
	sta pickchar
	rts
	
key_esc
	lda $ff19
	eor #$33
	sta $ff19	
 db $ab,$00		; LAX #$00
	lda $0800,x
	sta $2800,x
	lda $0900,x
	sta $2900,x
	lda $0a00,x
	sta $2a00,x
	lda $0ae8,x
	sta $2ae8,x
	lda $0c00,x
	sta $2c00,x
	lda $0d00,x
	sta $2d00,x
	lda $0e00,x
	sta $2e00,x
	lda $0ee8,x
	sta $2ee8,x
	inx
	bne key_esc+10
	lda $ff19
	eor #$33
	sta $ff19
	rts
			
key_h
	lda $ff15
	sta tmp
	lda #$21
	sta $ff15
	lda #$1b
	sta $ff06
	lda #$c4
	sta $ff12
	lda #$d0
	sta $ff13
	lda #$1f
	sta $ff14
	lda #$ff
	sta $ff0c
	sta $ff0d
waitforh
	jsr loop_kdelay
	lda #$f7
	sta $fd30
	sta $ff08
	lda $ff08
	cmp #$df
	bne waitforh
	lda #$5b
	sta $ff06
	lda #$c0
	sta $ff12
	lda #chars >> 8
	sta $ff13
	lda #$0f
	sta $ff14
	lda tmp
	sta $ff15
	rts
	;
tmp
 db $ee
 
flag
 db $7f
 
cntr
 db $00,$ff  

keyb1
 db $fe,$fe,$fe,$fe,$fe,$fc,$fc,$fc,$fc,$ef,$ef,$ed,$ed,$fe,$fc,$7f
 db $bf,$bf,$df,$df,$7f,$fd,$fb,$fb,$f7,$bd,$bd,$dd,$dd,$7f
 db $7f,$7f,$fd,$fd,$fb,$fb,$f7,$f7,$7d,$f7,$bf
 
keyb2
 db $fd,$ef,$df,$bf,$f7,$6f,$5f,$3f,$77,$fe,$f7,$7e,$77,$fe,$7e,$ef
 db $fe,$f7,$fe,$f7,$dd,$ef,$7f,$ef,$7f,$7e,$77,$7e,$77,$fd
 db $fe,$f7,$fe,$f7,$fe,$f7,$fe,$f7,$7d,$df,$ef
 
keyb2x
 db key_return & 255, key_return >> 8, key_f1 & 255, key_f1 >> 8, key_f2 & 255, key_f2 >> 8, key_f3 & 255, key_f3 >> 8
 db key_help & 255, key_help >> 8, key_f4 & 255, key_f4 >> 8, key_f5 & 255, key_f5 >> 8, key_f6 & 255, key_f6 >> 8, key_f7 & 255, key_f7 >> 8
 db key_9 & 255, key_9 >> 8, key_0 & 255, key_0 >> 8, key_sh9 & 255, key_sh9 >> 8, key_sh0 & 255, key_sh0 >> 8
 db key_instdel & 255, key_instdel >>8, key_shinstdel & 255, key_shinstdel >>8, key_space & 255, key_space >> 8
 db key_left & 255, key_left >> 8, key_right & 255, key_right >> 8, key_down & 255, key_down >> 8, key_up & 255, key_up >> 8
 db key_cbmhome & 255, key_cbmhome >> 8, key_z & 255, key_z >> 8, key_x & 255, key_x >> 8, key_c & 255, key_c >> 8
 db key_v & 255, key_v >> 8, key_shleft & 255, key_shleft >> 8, key_shright & 255, key_shright >> 8
 db key_shdown & 255, key_shdown >> 8, key_shup & 255, key_shup >> 8, key_home & 255, key_home >> 8
 db key_1 & 255, key_1 >> 8, key_2 & 255, key_2 >> 8, key_3 & 255, key_3 >> 8, key_4 & 255, key_4 >> 8
 db key_5 & 255, key_5 >> 8, key_6 & 255, key_6 >> 8, key_7 & 255, key_7 >> 8, key_8 & 255, key_8 >> 8
 db key_shhome & 255, key_shhome >> 8, key_h & 255, key_h >> 8, key_esc & 255, key_esc >> 8

	org $1800
	include "helpscreen.asm"
 
	org $2000
chars
	include "superdither_charset.asm"
	